/* basicfunc.c, (basic) functions, used by more than one cgi

 * Copyright (C) 2001,2002 M.C. Bezemer
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <cgic.h>

#include "untag.h"
#include "findvalue.h"
#include "session.h"
#include "urlencode.h"
#include "basicfunc.h"


void OpenTAS_send_pagestart(int nocache, int expire, int error_title,
			    char *extra_head_stuff)
{
	cgiHeaderContentType("text/html");
	fprintf(cgiOut,	"<html>\n<head>\n");
	if (nocache) {
		fprintf(cgiOut,
			"<meta http-equiv=\"pragma\" content=\"no-cache\">\n"
			"<meta http-equiv=\"cache-control\" "
			"content=\"no-cache, must-revalidate\">\n");
	}
	if (expire) {
		fprintf(cgiOut,
			"<meta http-equiv=\"expires\" "
			"content=\"Mon, 26 Jul 1997 05:00:00 GMT\">\n");
	}
	if (extra_head_stuff!=NULL) {
		fprintf(cgiOut, "%s\n", extra_head_stuff);
	}
	if (error_title) {
		fprintf(cgiOut, "<title>OpenTAS - Fout!</title>\n");
	} else {
		fprintf(cgiOut, "<title>OpenTAS</title>\n");
	}
	fprintf(cgiOut,
		"</head>\n"
		"<body bgcolor=\"#fffff3\">\n"
		"<img src=\"opentas.png\" border=\"0\" alt=\"OpenTAS\">\n"
		"<hr noshade width=\"75%%\" align=\"left\">\n<p>\n");

	return;
}


void OpenTAS_errormsg(char *string)
{
	OpenTAS_send_pagestart(1,1,1,NULL);
	fprintf(cgiOut,
		"Er is een fout opgetreden:<br>\n"
		"%s\n"
		"</body>\n</html>\n",string);
	exit(0);
}


void OpenTAS_get_session_data(char *session_id, char **studienummer,
			      char **wachtwoord, char **tas_stud_id)
/* OpenTAS wrapper for get_session_data(), 
 * generates HTML output and exits on failure
 */
{
	if (get_session_data(session_id, studienummer,
			     wachtwoord, tas_stud_id)!=0) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"Onbekende of verlopen sessie. "
			"<a href=\"index.html\">Log opnieuw in.</a>\n"
			"</body>\n</html>\n");
		exit(0);
	}
	return;
}


void OpenTAS_check_foutnr(char *session_id)
/* checks TAS cgi output for a "foutnr" field; generates HTML and exits
 * if found.
 */
{
	if (findfirstvalue("foutnr")!=NULL) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		OpenTAS_navigate(session_id);
		fprintf(cgiOut,
			"Er is een fout opgetreden.<br>\n"
			"TAS-foutmelding:<br>"
			"%s: ", untag(findfirstvalue("foutnr")));
		fprintf(cgiOut,
			"%s\n"
			"</body>\n</html>\n",
			untag(findfirstvalue("foutmelding")));
		exit(0);
	}
	return;
}


void OpenTAS_check_status_1(char *session_id)
/* checks TAS cgi output for a "status" field; generates HTML and exits
 * if status doesn't contain "1".
 */
{
	char *status=NULL;
	status=findfirstvalue("status");
	if ((status==NULL)||(strcmp(status,"1")!=0)) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		OpenTAS_navigate(session_id);
		fprintf(cgiOut,
			"Er is een fout opgetreden.<br>\n"
			"De TAS-server heeft de opdracht niet "
			"geaccepteerd.\n"
			"</body>\n</html>\n");
		exit(0);
	}
	return;
}


void OpenTAS_navigate(char *session_id)
/* Outputs navigation buttons on top of the page */
{
	fprintf(cgiOut,
		"<table border=\"0\" rules=\"none\" frame=\"void\" "
		"cellspacing=\"3\">\n"
		"<tr>\n"
		"<td>\n"
		"<form action=\"pers_geg.cgi\" method=\"post\">\n"
		"<input type=\"hidden\" name=\"session_id\" "
		"value=\"%1$s\">\n"
		"<input type=\"submit\" value=\"Persoonsgegevens\">\n"
		"</form>\n"
		"</td>\n"
		"<td>\n"
		"<form action=\"aanmeld_sel.cgi\" method=\"post\">\n"
		"<input type=\"hidden\" name=\"session_id\" "
		"value=\"%1$s\">\n"
		"<input type=\"submit\" value=\"Selectie / Aanmelden\">\n"
		"</form>\n"
		"</td>\n"
		"<td>\n"
		"<form action=\"overzicht.cgi\" method=\"post\">\n"
		"<input type=\"hidden\" name=\"session_id\" "
		"value=\"%1$s\">\n"
		"<input type=\"submit\" value=\"Overzicht / Afmelden\">\n"
		"</form>\n"
		"</td>\n"
		"<td>\n"
		"<form action=\"logout.cgi\" method=\"post\">\n"
		"<input type=\"hidden\" name=\"session_id\" "
		"value=\"%1$s\">\n"
		"<input type=\"submit\" value=\"Uitloggen\">\n"
		"</form>\n"
		"</td>\n"
		"</tr>\n"
		"</table>\n"
		"<p>\n",
		untag(session_id));
	return;
}

void OpenTAS_back_to_search_results(char *session_id,
				    char *ot_evn_soort,
				    char *ot_tas_ople_id,
				    char *ot_vak_type,
				    char *ot_tas_vak_id,
				    char *ot_oe_code,
				    char *ot_van_datum,
				    char *ot_tot_datum)
/* Outputs a button (within a table) to return to the results of
 * the last search operation (as returned by aanmeld_res.cgi)
 */
{
	fprintf(cgiOut,
		"<form action=\"aanmeld_res.cgi\" method=\"post\">\n"
		"<table>\n<tr>\n<td>\n"
		"<input type=\"hidden\" name=\"session_id\" "
		"value=\"%s\">\n",
		untag(session_id));
	fprintf(cgiOut,
		"<input type=\"hidden\" name=\"evn_soort\" "
		"value=\"%s\">\n",
		untag(ot_evn_soort));
	fprintf(cgiOut,
		"<input type=\"hidden\" name=\"tas_ople_id\" "
		"value=\"%s\">\n",
		untag(ot_tas_ople_id));
	fprintf(cgiOut,
		"<input type=\"hidden\" name=\"vak_type\" "
		"value=\"%s\">\n",
		untag(ot_vak_type));
	fprintf(cgiOut,
		"<input type=\"hidden\" name=\"tas_vak_id\" "
		"value=\"%s\">\n",
		untag(ot_tas_vak_id));
	fprintf(cgiOut,
		"<input type=\"hidden\" name=\"oe_code\" "
		"value=\"%s\">\n",
		untag(ot_oe_code));
	fprintf(cgiOut,
		"<input type=\"hidden\" name=\"van_datum\" "
		"value=\"%s\">\n",
		untag(ot_van_datum));
	fprintf(cgiOut,
		"<input type=\"hidden\" name=\"tot_datum\" "
		"value=\"%s\">\n",
		untag(ot_tot_datum));
	fprintf(cgiOut,
		"<br>"
		"<input type=\"submit\" value=\"Terug naar zoekresultaten\">\n"
		"</td>\n</tr>\n</table>\n"
		"</form>\n");

	return;
}

void OpenTAS_show_personal_data(char *session_id)
/* BIG FAT WARNING: Requires a prior SelStd TAS query! */
{
	fprintf(cgiOut,
		"<table border=\"0\" rules=\"none\" frame=\"void\">\n"
		"<tr>\n<td>Naam:</td>\n<td>%s</td>\n",
		untag(findfirstvalue("naam")));
	fprintf(cgiOut,
		"</tr><tr>\n<td>Geslacht:</td>\n<td>%s</td>\n",
		untag(findfirstvalue("geslacht")));
	fprintf(cgiOut,
		"</tr><tr>\n<td>Opleiding(en):</td>\n<td>%s</td>\n",
		untag(findfirstvalue("opleiding")));
	fprintf(cgiOut,
		"</tr><tr>\n<td>Studienumer:</td>\n<td>%s</td>\n",
		untag(findfirstvalue("studienummer")));
	fprintf(cgiOut,
		"</tr><tr>\n<td>Adres:</td>\n<td>%s</td>\n",
		untag(findfirstvalue("adres")));
	fprintf(cgiOut,
		"</tr><tr>\n<td>Woonplaats:</td>\n<td>%s</td>\n",
		untag(findfirstvalue("woonplaats")));
	fprintf(cgiOut,
		"</tr><tr>\n<td>Email:</td>\n<td>%s</td>\n",
		untag(findfirstvalue("email")));
	fprintf(cgiOut,
		"</tr><tr>\n<td>Wachtwoord aanwezig:</td>\n<td>%s</td>\n",
		untag(findfirstvalue("password")));
	/* in applet wordt gecheckt op 'Ja'; geen 'Ja' betekent 'Nee'.
	 * maar of er ook 'Nee' in staat als het niet 'Ja' is???....
	 * Maar goed, als het geen 'Ja' is, kun je het niet gebruiken om
	 * in te loggen, dus die vraag is hier niet zo interessant.
	 */
	fprintf(cgiOut,
		"</tr>\n</table>\n<p>\n");

	fprintf(cgiOut,
		"<table border=\"0\" rules=\"none\" frame=\"void\" "
		"cellspacing=\"3\">\n<tr>\n<td>\n"
		"<form action=\"update_email.cgi\" method=\"post\">\n"
		"<input type=\"hidden\" name=\"session_id\" value=\"%s\">\n"
		"<input type=\"submit\" value=\"Verander email-adres\">\n"
		"</form>\n</td>\n",
		untag(session_id));

	fprintf(cgiOut,
		"<td>\n<form action=\"update_password.cgi\" method=\"post\">\n"
		"<input type=\"hidden\" name=\"session_id\" value=\"%s\">\n"
		"<input type=\"submit\" value=\"Verander wachtwoord\">\n"
		"</form>\n</td>\n<td>\n</td>\n</tr>\n</table>\n",
		untag(session_id));

	return;
}

void OpenTAS_write_URL(char *tasvalue, char *prefix, char *suffix)
{
	char *temp=NULL;

	if (tasvalue==NULL) {
		return;
	}

	if (prefix==NULL) {
		prefix="";
	}
	if (suffix==NULL) {
		suffix="";
	}

	temp=findfirstvalue(tasvalue);
	if (temp==NULL) {
		return;
	}
	if (strcmp(temp," ")==0) {
		return;
	}

	fprintf(cgiOut,"%s",prefix);
	fprintf(cgiOut,"<a href=\"");
	if (strlen(temp)<7) {
		fprintf(cgiOut,"http://%s\">",untag(temp));
		fprintf(cgiOut,"http://%s",untag(temp));
	} else {
		if (strncmp(temp,"http://",7)==0) {
			fprintf(cgiOut,"%s\">",untag(temp));
			fprintf(cgiOut,"%s",untag(temp));
		} else {
			fprintf(cgiOut,"http://%s\">",untag(temp));
			fprintf(cgiOut,"http://%s",untag(temp));
		}
	}
	fprintf(cgiOut,"</a>");
	fprintf(cgiOut,"%s",suffix);

	return;
}

void OpenTAS_malloc_error(char *session_id)
{
	OpenTAS_send_pagestart(1,1,1,NULL);
	OpenTAS_navigate(session_id);
	fprintf(cgiOut,
		"Er is een fout opgetreden:<br>\n"
		"OpenTAS server heeft onvoldoende vrij geheugen voor deze"
		" taak.<br>\n</body>\n</html>\n");
	exit(0);
}

void OpenTAS_invalid_tasdata(char *session_id)
{
	OpenTAS_send_pagestart(1,1,1,NULL);
	OpenTAS_navigate(session_id);
	fprintf(cgiOut,
		"Er is een fout opgetreden:<br>\n"
		"Gegevens van TAS server onvolledig of ongeldig.<br>\n"
		"</body>\n</html>\n");
	exit(0);
}

