/* findvalue.c, functions to handle the data returned by the TAS cgi

 * Copyright (C) 2001,2002 M.C. Bezemer
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "urldecode.h"
#include "findvalue.h"

struct tasvalue {
	char *key;
	char *value;
	struct tasvalue *next;
};

struct tasvalue *fv_tasvalues=NULL;
struct tasvalue *fv_tasvaluepointer=NULL;
struct tasvalue *fv_tasvalues_saved=NULL;

void freetasvalues(void)
{
	struct tasvalue *temptasvalues=NULL;

	while (fv_tasvalues!=NULL) {
		free(fv_tasvalues->key);
		free(fv_tasvalues->value);
		temptasvalues=fv_tasvalues;
		fv_tasvalues=fv_tasvalues->next;
		free(temptasvalues);
	}
	return;
}

int inittasresponse(char *tasstring)
{
	char *mytasstring=NULL;
	char *currentpos=NULL;
	char *amppointer=NULL;
	char *eqpointer=NULL;
	struct tasvalue *mytasvalue=NULL;

	if (tasstring==NULL) {
		return -1;
	}
	if (strlen(tasstring)==0) {
		return 0;
	}
	mytasstring=strdup(tasstring);
	if (mytasstring==NULL) {
		return -2;
	}

	currentpos=mytasstring;
	freetasvalues();
	fv_tasvalues=malloc(sizeof(struct tasvalue));
	if (fv_tasvalues==NULL) {
		return -3;
	}
	fv_tasvalues->next=fv_tasvalues;
	mytasvalue=fv_tasvalues;

	amppointer=currentpos;
	while (amppointer!=NULL) {
		mytasvalue=mytasvalue->next;
		amppointer=strchr(currentpos,'&');
		if (amppointer!=NULL) {
			*amppointer='\0';
		}
		eqpointer=strchr(currentpos,'=');
		if (eqpointer==NULL) {
			mytasvalue->value=strdup("");
		} else {
			*eqpointer='\0';
			mytasvalue->value=strdup(urldecode(eqpointer+1));
		}
		if (mytasvalue->value==NULL) {
			return -4;
		}
		mytasvalue->key=strdup(urldecode(currentpos));
		if (mytasvalue->key==NULL) {
			return -5;
		}
		mytasvalue->next=malloc(sizeof(struct tasvalue));
		if (mytasvalue->next==NULL) {
			return -6;
		}
		if (amppointer!=NULL) {
			currentpos=amppointer+1;
		}
	}
	free(mytasvalue->next);
	mytasvalue->next=NULL;

	free(mytasstring);

	reset_tasvaluepointer();

	return 0;
}


void reset_tasvaluepointer(void)
{
	fv_tasvaluepointer=fv_tasvalues;
	return;
}


char *findnextvalue(char *key)
{
	char *returnvalue;
	while (fv_tasvaluepointer!=NULL) {
		if (strcmp(fv_tasvaluepointer->key,key)==0) {
			returnvalue=fv_tasvaluepointer->value;
			fv_tasvaluepointer=fv_tasvaluepointer->next;
			return returnvalue;
		}
		fv_tasvaluepointer=fv_tasvaluepointer->next;
	}
	return NULL;
}


char *findfirstvalue(char *label)
{
	reset_tasvaluepointer();
	return findnextvalue(label);
}


void save_tree(void)
{
	struct tasvalue *temptasvalues=NULL;
	temptasvalues=fv_tasvalues;
	fv_tasvalues=fv_tasvalues_saved;
	fv_tasvalues_saved=temptasvalues;
	freetasvalues();
	return;
}

void restore_tree(void)
{
	freetasvalues();
	fv_tasvalues=fv_tasvalues_saved;
	return;
}
