/* login.c, checks username and password with TAS server and creates
 * an OpenTAS session

 * Copyright (C) 2001,2002 M.C. Bezemer
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include <cgic.h>

#include "untag.h"
#include "urlencode.h"
#include "urldecode.h"
#include "findvalue.h"
#include "tasquery.h"
#include "session.h"
#include "basicfunc.h"


int cgiMain(void)
{
	char studienummer[15];
	char wachtwoord[200];
	cgiFormResultType status;
	int status_error=0;
	char *tasresponse=NULL;
	size_t tasresponse_len=0;
	char *tas_stud_id=NULL;
	char *session_id=NULL;

	status=cgiFormString("studienummer",studienummer,15);
	if (status!=cgiFormSuccess) {
		status_error=1;
	}
	status=cgiFormString("wachtwoord",wachtwoord,200);
	if (status!=cgiFormSuccess) {
		status_error=1;
	}

	if (status_error) {
		OpenTAS_send_pagestart(1,1,0,NULL);
	        fprintf(cgiOut,
			"<form action=\"login.cgi\" method=\"post\">"
			"<table border=\"0\" rules=\"none\" frame=\"void\""
			" cellspacing=\"10\">\n<tr>\n"
			"<td>Studienummer:</td>\n"
			"<td><input type=\"text\" name=\"studienummer\"></td>"
			"\n</tr>\n<tr>\n"
			"<td>Wachtwoord:</td>\n"
			"<td><input type=\"password\" name=\"wachtwoord\">"
			"</td>\n</tr>\n<tr>\n"
			"<td colspan=\"2\" align=\"center\">\n"
			"<input type=\"submit\" value=\"Inloggen\">\n"
			"</td>\n</tr>\n</table>\n"
			"</form>\n"
	                "</body>\n</html>\n");
		exit(0);
	}

	tas_query_init();
	tas_query_add("USERNAME=",0);
	tas_query_add(studienummer,1);
	tas_query_add("&PASSWORD=",0);
	tas_query_add(wachtwoord,1);
	tas_query_add("&request=ChkPwd",0);

	if (do_tas_query(&tasresponse,&tasresponse_len)!=0) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"Communicatie met TAS-server mislukt (%s)\n<p>\n",
			untag(tasresponse));
	        fprintf(cgiOut,
			"<form action=\"login.cgi\" method=\"post\">"
			"<table border=\"0\" rules=\"none\" frame=\"void\""
			" cellspacing=\"10\">\n<tr>\n"
			"<td>Studienummer:</td>\n"
			"<td><input type=\"text\" name=\"studienummer\"></td>"
			"\n</tr>\n<tr>\n"
			"<td>Wachtwoord:</td>\n"
			"<td><input type=\"password\" name=\"wachtwoord\">"
			"</td>\n</tr>\n<tr>\n"
			"<td colspan=\"2\" align=\"center\">\n"
			"<input type=\"submit\" value=\"Inloggen\">\n"
			"</td>\n</tr>\n</table>\n"
			"</form>\n"
			"</body>\n</html>\n");
		exit(0);
	}

	inittasresponse(tasresponse);

	if (findfirstvalue("foutnr")!=NULL) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"TAS-foutmelding:<br>"
			"%s: ", untag(findfirstvalue("foutnr")));
		fprintf(cgiOut,
			"%s\n<p>\n",
			untag(findfirstvalue("foutmelding")));
	        fprintf(cgiOut,
			"<form action=\"login.cgi\" method=\"post\">"
			"<table border=\"0\" rules=\"none\" frame=\"void\""
			" cellspacing=\"10\">\n<tr>\n"
			"<td>Studienummer:</td>\n"
			"<td><input type=\"text\" name=\"studienummer\"></td>"
			"\n</tr>\n<tr>\n"
			"<td>Wachtwoord:</td>\n"
			"<td><input type=\"password\" name=\"wachtwoord\">"
			"</td>\n</tr>\n<tr>\n"
			"<td colspan=\"2\" align=\"center\">\n"
			"<input type=\"submit\" value=\"Inloggen\">\n"
			"</td>\n</tr>\n</table>\n"
			"</form>\n"
			"</body>\n</html>\n");
		exit(0);
	}

	if (findfirstvalue("tas_stud_id")==NULL) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		fprintf(cgiOut,
			"Er is geen tas_stud_id voor deze student.<br>\n"
			"Neem contact op met de TAS-beheerder.\n"
			"</body>\n</html>\n");
		exit(0);
	}
	tas_stud_id=strdup(findfirstvalue("tas_stud_id"));

	if (save_session_data(studienummer,wachtwoord,tas_stud_id,&session_id)!=0) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		fprintf(cgiOut,
			"Er kan geen sessie meer gemaakt worden!<br>\n"
			"Neem contact op met de OpenTAS-beheerder.\n"
			"</body>\n</html>\n");
		exit(0);
	}

	tas_query_init();
	tas_query_add("USERNAME=",0);
	tas_query_add(studienummer,1);
	tas_query_add("&PASSWORD=",0);
	tas_query_add(wachtwoord,1);
	tas_query_add("&request=SelStd&tas_stud_id=",0);
	tas_query_add(tas_stud_id,1);

	if (do_tas_query(&tasresponse,&tasresponse_len)!=0) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		OpenTAS_navigate(session_id);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"Communicatie met TAS-server mislukt (%s)<br>\n"
			"</body>\n</html>\n", untag(tasresponse));
		exit(0);
	}
	inittasresponse(tasresponse);
	free(tasresponse);

	OpenTAS_check_foutnr(session_id);

	OpenTAS_send_pagestart(1,1,0,NULL);
	OpenTAS_navigate(session_id);

	OpenTAS_show_personal_data(session_id);

	fprintf(cgiOut, "</body>\n</html>\n");

	return 0;
}
