/* overzicht.c, list all events currently signed up for

 * Copyright (C) 2001,2002 M.C. Bezemer
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include <cgic.h>

#include "untag.h"
#include "urlencode.h"
#include "urldecode.h"
#include "findvalue.h"
#include "tasquery.h"
#include "session.h"
#include "basicfunc.h"


struct ovz_struct {
	char *afmeld_mogelijk;		/* 1 */
	char *evn_soort;		/* 2 */
	char *tas_even_id;		/* 3 */
	char *oe_code;			/* 4 */
	char *vak_oms_ned;		/* 5 */
	char *evn_datum;		/* 6 */
	char *evn_datum_sort;		/* 7 */
	char *tijd;			/* 8 */
	char *datum_stop_terugtrek;	/* 9 */
	char *aanmeld_status_tr;	/* 10 */
};


int datetimecompare(struct ovz_struct *a, struct ovz_struct *b)
{
	int i;

	i=strcmp(a->evn_datum_sort, b->evn_datum_sort);
	if (i==0) {
		i=strcmp(a->tijd, b->tijd);
	}
	return i;
}


int oecodecompare(struct ovz_struct *a, struct ovz_struct *b)
{
	int i;

	i=strcmp(a->oe_code, b->oe_code);

	return i;
}


int cgiMain(void)
{
	char session_id[35];
	char *studienummer=NULL;
	char *wachtwoord=NULL;
	char *tas_stud_id=NULL;
	int ot_sort_mode=1;
	cgiFormResultType status;
	char *tasresponse=NULL;
	size_t tasresponse_len=0;
	char *temppointer=NULL;
	int ovz_counter=0;
	struct ovz_struct *ovz_list=NULL;
	int i;

	status=cgiFormString("session_id",session_id,35);
	if (status!=cgiFormSuccess) {
		strcpy(session_id,"NULL-session");
	}

	OpenTAS_get_session_data(session_id, &studienummer,
				 &wachtwoord, &tas_stud_id);

	cgiFormIntegerBounded("ot_sort_mode",&ot_sort_mode,0,2,1);
	/* sort mode 0: don't sort
	 * sort mode 1: evn_date_sort+time [default]
	 * sort mode 2: oe_code
	 */

	tas_query_init();
	tas_query_add("USERNAME=",0);
	tas_query_add(studienummer,1);
	tas_query_add("&PASSWORD=",0);
	tas_query_add(wachtwoord,1);
	tas_query_add("&request=SelOvr&tas_stud_id=",0);
	tas_query_add(tas_stud_id,1);

	if (do_tas_query(&tasresponse,&tasresponse_len)!=0) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		OpenTAS_navigate(session_id);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"Communicatie met TAS-server mislukt (%s)<br>\n"
			"</body>\n</html>\n", untag(tasresponse));
		exit(0);
	}
	inittasresponse(tasresponse);

	OpenTAS_check_foutnr(session_id);

	reset_tasvaluepointer();
	ovz_counter=0;
	while (findnextvalue("afmeld_mogelijk")!=NULL) {
		ovz_counter++;
	}
	ovz_list=malloc((ovz_counter+1)*sizeof(struct ovz_struct));
	if (ovz_list==NULL) {
		OpenTAS_malloc_error(session_id);
	}

	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("afmeld_mogelijk");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].afmeld_mogelijk=strdup(temppointer);
		if (ovz_list[i].afmeld_mogelijk==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("evn_soort");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].evn_soort=strdup(temppointer);
		if (ovz_list[i].evn_soort==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("tas_even_id");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].tas_even_id=strdup(temppointer);
		if (ovz_list[i].tas_even_id==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("oe_code");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].oe_code=strdup(temppointer);
		if (ovz_list[i].oe_code==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("vak_oms_ned");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].vak_oms_ned=strdup(temppointer);
		if(ovz_list[i].vak_oms_ned==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("evn_datum");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].evn_datum=strdup(temppointer);
		if(ovz_list[i].evn_datum==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("evn_datum_sort");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].evn_datum_sort=strdup(temppointer);
		if(ovz_list[i].evn_datum_sort==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("tijd");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].tijd=strdup(temppointer);
		if(ovz_list[i].tijd==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("datum_stop_terugtrek");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].datum_stop_terugtrek=strdup(temppointer);
		if(ovz_list[i].datum_stop_terugtrek==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}
	reset_tasvaluepointer();
	for (i=0;i<ovz_counter;i++) {
		temppointer=findnextvalue("aanmeld_status_tr");
		if (temppointer==NULL) {
			OpenTAS_invalid_tasdata(session_id);
		}
		ovz_list[i].aanmeld_status_tr=strdup(temppointer);
		if(ovz_list[i].aanmeld_status_tr==NULL) {
			OpenTAS_malloc_error(session_id);
		}
	}

	if (ot_sort_mode==1) {
		qsort(ovz_list,
		      ovz_counter,
		      sizeof(struct ovz_struct),
		      (void*)datetimecompare);
	} else if (ot_sort_mode==2) {
		qsort(ovz_list,
		      ovz_counter,
		      sizeof(struct ovz_struct),
		      (void*)oecodecompare);
	}

	OpenTAS_send_pagestart(1,1,0,NULL);
	OpenTAS_navigate(session_id);

	fprintf(cgiOut,
		"Klik op 'Detail' voor details over de betreffende "
		"activiteit of op 'Afmelden' om direct af te melden.\n"
		"<br>Mogelijk is 'Afmelden' vervangen door 'Definitief'. "
		"In dat geval is afmelden niet meer mogelijk.\n<p>\n"
		"<table border=\"0\" rules=\"none\" frame=\"void\" "
		"cellspacing=\"2\" cellpadding=\"1\">\n<tr>\n<td>&nbsp;");
	fprintf(cgiOut,"</td><td>Activiteit&nbsp;</td><td>");
	if (ot_sort_mode==2) {
		fprintf(cgiOut,"<b>Vakcode</b>");
	} else {
		fprintf(cgiOut,
			"<a href=\"overzicht.cgi?session_id=%s&ot_sort_mode=2"
			"\">Vakcode</a>",
			urlencode(session_id));
	}
	fprintf(cgiOut,"&nbsp;</td><td>Omschrijving&nbsp;</td><td>");
	if (ot_sort_mode==1) {
		fprintf(cgiOut,"<b>Datum</b>&nbsp;</td><td><b>Tijd</b>&nbsp;");
	} else {
		fprintf(cgiOut,
			"<a href=\"overzicht.cgi?session_id=%s&ot_sort_mode=1"
			"\">Datum</a></td><td>",
			urlencode(session_id));
		fprintf(cgiOut,
			"<a href=\"overzicht.cgi?session_id=%s&ot_sort_mode=1"
			"\">Tijd</a>&nbsp;",
			urlencode(session_id));
	}
	fprintf(cgiOut,"</td><td>Status&nbsp;</td><td>&nbsp;</td></tr>\n");

	for (i=0;i<ovz_counter;i++) {
		fprintf(cgiOut,
			"<tr>\n<td><a href=\"overzicht_det.cgi?session_id=%s&",
			urlencode(session_id));
		fprintf(cgiOut,"evn_soort=%s&",
			urlencode(ovz_list[i].evn_soort));
		fprintf(cgiOut,"tas_even_id=%s",
			urlencode(ovz_list[i].tas_even_id));
		fprintf(cgiOut,"\">Detail</a>&nbsp;</td><td>");

		fprintf(cgiOut,"%s&nbsp;</td><td>",
				untag(ovz_list[i].evn_soort));
		fprintf(cgiOut,"%s&nbsp;</td><td>",
				untag(ovz_list[i].oe_code));
		fprintf(cgiOut,"%s&nbsp;</td><td nowrap>",
				untag(ovz_list[i].vak_oms_ned));
		fprintf(cgiOut,"<nobr>%s&nbsp;</nobr></td><td nowrap>",
				untag(ovz_list[i].evn_datum));
		fprintf(cgiOut,"<nobr>%s&nbsp;</nobr></td><td>",
				untag(ovz_list[i].tijd));
		fprintf(cgiOut,"%s&nbsp;</td><td>",
				untag(ovz_list[i].aanmeld_status_tr));
		if (strcmp(ovz_list[i].afmeld_mogelijk,"J")==0) {
			fprintf(cgiOut,
				"<a href=\"overzicht_afmtry.cgi?session_id=%s&",
				urlencode(session_id));
			fprintf(cgiOut,"evn_soort=%s&",
				urlencode(ovz_list[i].evn_soort));
			fprintf(cgiOut,"tas_even_id=%s",
				urlencode(ovz_list[i].tas_even_id));
			fprintf(cgiOut,"\">Afmelden</a>&nbsp;</td></tr>\n");
		} else {
			fprintf(cgiOut,"Definitief&nbsp;</td></tr>\n");
		}
	}

	fprintf(cgiOut,"</table>\n<p>\n</body>\n</html>\n");

	return 0;
}
