/* overzicht_afm.c, try to withdraw from an event previously signed up for

 * Copyright (C) 2001,2002 M.C. Bezemer
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <cgic.h>

#include "untag.h"
#include "urlencode.h"
#include "urldecode.h"
#include "findvalue.h"
#include "tasquery.h"
#include "session.h"
#include "basicfunc.h"


int cgiMain(void)
{
	char session_id[35];
	char *studienummer=NULL;
	char *wachtwoord=NULL;
	char *tas_stud_id=NULL;
	char evn_soort[100];
	char tas_even_id[100];
	cgiFormResultType status;
	char *tasresponse=NULL;
	size_t tasresponse_len=0;
	int status_error=0;
	char *afm_closedate=NULL;
	int afm_closeday=0;
	int afm_closemonth=0;
	int afm_closeyear=0;
	char *aanm_closedate=NULL;
	int aanm_closeday=0;
	int aanm_closemonth=0;
	int aanm_closeyear=0;
	time_t currtime=0;
	struct tm *currtm=NULL;

	status=cgiFormString("session_id",session_id,35);
	if (status!=cgiFormSuccess) {
		strcpy(session_id,"NULL-session");
	}

	OpenTAS_get_session_data(session_id, &studienummer,
				 &wachtwoord, &tas_stud_id);

	status=cgiFormString("tas_even_id",tas_even_id,100);
	if (status!=cgiFormSuccess) {
		status_error+=1;
	}
	status=cgiFormString("evn_soort",evn_soort,100);
	if (status!=cgiFormSuccess) {
		status_error+=2;
	}

	if (status_error) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		OpenTAS_navigate(session_id);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"Ongeldig tas_even_id of ongeldige evn_soort.<br>\n"
			"</body>\n</html>\n");
		exit(0);
	}

	tas_query_init();
	tas_query_add("USERNAME=",0);
	tas_query_add(studienummer,1);
	tas_query_add("&PASSWORD=",0);
	tas_query_add(wachtwoord,1);
	tas_query_add("&request=EvnDet&evn_soort=",0);
	tas_query_add(evn_soort,1);
	tas_query_add("&tas_even_id=",0);
	tas_query_add(tas_even_id,1);
	tas_query_add("&tas_stud_id=",0);
	tas_query_add(tas_stud_id,1);

	if (do_tas_query(&tasresponse,&tasresponse_len)!=0) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		OpenTAS_navigate(session_id);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"Communicatie met TAS-server mislukt (%s)<br>\n"
			"</body>\n</html>\n", untag(tasresponse));
		exit(0);
	}
	inittasresponse(tasresponse);

	OpenTAS_check_foutnr(session_id);

	status_error=0;

	afm_closedate=untag(findfirstvalue("datum_stop_terugtrek"));
	if (sscanf(afm_closedate,"%2d-%2d-%4d",
		   &afm_closeday,&afm_closemonth,&afm_closeyear)!=3) {
		//ongeldige datum, mogelijk niet meer afmelden/aanmelden
		status_error|=1;
	}

	aanm_closedate=untag(findfirstvalue("datum_stop_aanmeld"));
	if (sscanf(aanm_closedate,"%2d-%2d-%4d",
		   &aanm_closeday,&aanm_closemonth,&aanm_closeyear)!=3) {
		//ongeldige datum, mogelijk niet meer aanmelden
		status_error|=2;
	}

	currtime=time(NULL);
	currtm=localtime(&currtime);

	if (!(status_error&1)) {
		if ((currtm->tm_year+1900)>afm_closeyear) {
			// afmelden kan niet meer
			status_error|=4;
		} else if ((currtm->tm_year+1900)==afm_closeyear) {
			if ((currtm->tm_mon+1)>afm_closemonth) {
				// afmelden kan niet meer
				status_error|=4;
			} else if ((currtm->tm_mon+1)==afm_closemonth) {
				if (currtm->tm_mday>afm_closeday) {
					// afmelden kan niet meer
					status_error|=4;
				}
			}
		}
	}

	if (!(status_error&2)) {
		// als status_error nu nog 0 dan afmelden nog mogelijk
		if ((currtm->tm_year+1900)>aanm_closeyear) {
			// aanmelden kan niet meer
			status_error|=8;
		} else if ((currtm->tm_year+1900)==aanm_closeyear) {
			if ((currtm->tm_mon+1)>aanm_closemonth) {
				// aanmelden kan niet meer
				status_error|=8;
			} else if ((currtm->tm_mon+1)==aanm_closemonth) {
				if (currtm->tm_mday>aanm_closeday) {
					// aanmelden kan niet meer
					status_error|=8;
				}
			}
		}
	}

	if (status_error) {
		// foutmelding geven en link naar overzicht_afm
		OpenTAS_send_pagestart(1,1,0,NULL);
		OpenTAS_navigate(session_id);

		fprintf(cgiOut, "LET OP!!<br>\n&nbsp;\n<br>\n");

		if (status_error&1) {
			fprintf(cgiOut,
				"Uiterste afmeld-datum is onbekend.<br>\n"
				"Afmelden is misschien niet mogelijk.<br>\n");
		}
		if (status_error&2) {
			fprintf(cgiOut,
				"Uiterste aanmeld-datum is onbekend.<br>\n"
				"Opnieuw aanmelden is misschien niet mogelijk."
				"<br>\n");
		}
		if (status_error&4) {
			fprintf(cgiOut,
				"Uiterste afmeld-datum is verstreken.<br>\n"
				"Afmelden is waarschijnlijk niet mogelijk."
				"<br>\n");
		}
		if (status_error&8) {
			fprintf(cgiOut,
				"Uiterste aanmeld-datum is verstreken.<br>\n"
				"Opnieuw aanmelden is waarschijnlijk niet "
				"mogelijk.<br>\n");
		}
		fprintf(cgiOut,
			"&nbsp;<br>\n"
			"Druk op de knop 'Afmelden' om toch (te proberen) af "
			"te melden, of druk op de knop 'Terug naar het "
			"overzicht' om naar het overzicht terug te gaan.\n"
			"<p>\n");

		fprintf(cgiOut,
			"<table><tr><td>\n"
			"<form action=\"overzicht_afm.cgi\" method=\"post\">\n"
			"<input type=\"hidden\" name=\"session_id\" "
			"value=\"%s\">\n",
			untag(session_id));
		fprintf(cgiOut,
			"<input type=\"hidden\" name=\"evn_soort\" "
			"value=\"%s\">\n",
			untag(evn_soort));
		fprintf(cgiOut,
			"<input type=\"hidden\" name=\"tas_even_id\" "
			"value=\"%s\">\n",
			untag(tas_even_id));
		fprintf(cgiOut,
			"<br>"
			"<input type=\"submit\" value=\"Afmelden\">\n"
			"</form>\n"
			"</td></tr></table>\n");

		fprintf(cgiOut,
			"<table><tr><td>\n"
			"<form action=\"overzicht.cgi\" method=\"post\">\n"
			"<input type=\"hidden\" name=\"session_id\" "
			"value=\"%s\">\n",
			untag(session_id));
		fprintf(cgiOut,
			"<br>"
			"<input type=\"submit\" value=\"Terug naar het overzicht\">\n"
			"</form>\n"
			"</td></tr></table>\n");


		fprintf(cgiOut,
			"</body>\n</html>\n");
		exit(0);
	}


	// Afmelden en aanmelden nog mogelijk, dus no harm can be done here...
	// (Oude code van overzicht_afm.c)

	tas_query_init();
	tas_query_add("USERNAME=",0);
	tas_query_add(studienummer,1);
	tas_query_add("&PASSWORD=",0);
	tas_query_add(wachtwoord,1);
	tas_query_add("&request=EvnAfm&evn_soort=",0);
	tas_query_add(evn_soort,1);
	tas_query_add("&tas_even_id=",0);
	tas_query_add(tas_even_id,1);
	tas_query_add("&tas_stud_id=",0);
	tas_query_add(tas_stud_id,1);

	if (do_tas_query(&tasresponse,&tasresponse_len)!=0) {
		OpenTAS_send_pagestart(1,1,1,NULL);
		OpenTAS_navigate(session_id);
		fprintf(cgiOut,
			"Er is een fout opgetreden:<br>\n"
			"Communicatie met TAS-server mislukt (%s)<br>\n"
			"</body>\n</html>\n", untag(tasresponse));
		exit(0);
	}
	inittasresponse(tasresponse);

	OpenTAS_check_foutnr(session_id);
	OpenTAS_check_status_1(session_id);
	//Applet-foutnr: E_801

	OpenTAS_send_pagestart(1,1,0,NULL);
	OpenTAS_navigate(session_id);

	fprintf(cgiOut, "Je bent afgemeld.<br>\n");

	fprintf(cgiOut,
		"<table><tr><td>\n"
		"<form action=\"overzicht.cgi\" method=\"post\">\n"
		"<input type=\"hidden\" name=\"session_id\" "
		"value=\"%s\">\n",
		untag(session_id));
	fprintf(cgiOut,
		"<br>"
		"<input type=\"submit\" value=\"Terug naar het overzicht\">\n"
		"</form>\n"
		"</td></tr></table>\n");


	fprintf(cgiOut,
		"</body>\n</html>\n");

	return 0;
}
