/* urldecode.c, HTML URL decoding (like PHP's rawurldecode()-function)

 * Copyright (C) 2001,2002 M.C. Bezemer
 *
 * Licensed under the terms of the GNU General Public License.
 * ABSOLUTELY NO WARRANTY.
 * See the file `COPYING' in this directory.
 */


#include "urldecode.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>


int hexvalue(char c1, char c2)
{
	static char *hexvaluestring="0123456789abcdef";
	int returnvalue=0;
	int c1l;
	int c2l;

	c1l=tolower(c1);
	c2l=tolower(c2);

	if (isxdigit(c1l) && isxdigit(c2l)) {
		returnvalue+=16*(strchr(hexvaluestring,c1l)-hexvaluestring);
		returnvalue+=(strchr(hexvaluestring,c2l)-hexvaluestring);
	}
	return returnvalue;
}


void add_to_decstring(char *addition,
		      unsigned long l /*= length*/,
		      char **decstring,
		      unsigned long *decstringlen,
		      unsigned long *decstringpos )
{
	/* *decstringpos points to "final" \0 char in string, that will be
	   overwritten by first char of addition => pos == string length */
	if (*decstringpos + l + 1 > *decstringlen) {
		*decstringlen = *decstringpos + l + 2000;
		*decstring = realloc(*decstring, *decstringlen);
	}

	memcpy(*decstring + *decstringpos, addition, l);
	*decstringpos += l;
	(*decstring)[*decstringpos] = '\0';
}


char *urldecode(char *s)
/* Renders all %xx entities to normal characters. Returns pointer
   to string that is valid only until next call of _this_ function
   (i.e. uses different buffer than l() ).
   (i.e. DON'T USE TWICE WITHIN ONE PRINTF!!) */
{
	static char *decstring = NULL;
	unsigned long decstringlen = 0;
	unsigned long decstringpos = 0;
	register unsigned long slen;
	register unsigned long curpos = 0;
	unsigned long startpos = 0;
	register unsigned char c;
	char entity[20];

	if (s==NULL) {
		return NULL;
	}
	slen = strlen(s);
	free(decstring);
	decstring = NULL;
	decstringpos = 0;
	decstringlen = 0;

	while (curpos < slen) {
		c = s[curpos];

		while ((c!='%') && (c!='+') && curpos < slen) {
			curpos++;
			c = s[curpos];
		}

		if (curpos > startpos)
			add_to_decstring(s + startpos,
					 curpos - startpos,
					 &decstring,
					 &decstringlen,
					 &decstringpos);

		if (c=='+') {
			snprintf(entity,2," ");
			add_to_decstring(entity,
					 1,
					 &decstring,
					 &decstringlen,
					 &decstringpos);
			curpos-=2;
		} else if (curpos + 2 < slen ) {
			snprintf(entity,
				 2,
				 "%c",
				 hexvalue(s[curpos+1],s[curpos+2]));
			if (entity[0]!='\0') {
				add_to_decstring(entity,
						 strlen(entity),
						 &decstring,
						 &decstringlen,
						 &decstringpos);
			}
		}
		curpos+=3;
		startpos = curpos;
	}

	/* make sure string does exists and is terminated correctly */
	decstring = realloc(decstring, decstringpos + 1);
	decstring[decstringpos] = 0;

	return decstring;
}
